function varargout=runme(varargin)
%RUNME - test deck for ISSM nightly runs
%
%   Usage:
%      varargout=runme(varargin);
%
%   Examples:
%      runme;
%      runme({'ice'});
%      runme({'ice','cielo_serial'},{'absolute'});
%      md=runme({'cielo_parallel'});
%
%   See Also: UPDATEARCHIVE

% This file can be run to check that the current version of issm is giving 
% coherent results. This test deals with Pine Island Glacierfor a 2d model.
% Just run this file in Matlab, with a properly setup ISSM code. 
% The results of this test will indicate if there is a difference between current computations 
% and archived results.

% Errors  between archived results and the current version will get flagged if they are not within 
% a certain tolerance. The current tolerance is 10^-12. If you have good reasons to believe this 
% tolerance should be lowered (for example, if you are running single precision compilers?), feel 
% free to tweak the tolerance variable.

%check arguments
if (nargin>2 | nargout>1)
	help runme
	error('runme error message: bad usage');
end

%packages and misfits to be tested
if nargin==2,
	misfits=varargin{2};
else
	misfits={'absolute','relative','logarithmic'};
end
if nargin,
	packages=varargin{1};
else
	packages={'cielo_serial','cielo_parallel'};
end

%Initialize log message for nightly runs.
testname='PigControlMethodDragM2d_20';

%go through the solutions requested
for i=1:length(packages),
	package=packages{i};

	for j=1:length(misfits),
		misfit=misfits{j};

		%check package
		if ~(strcmpi(package,'cielo_serial') | strcmpi(package,'cielo_parallel'));
			disp(['package: ' package  ' in test: ' testname  ' not supported yet']);
			continue
		end

		%initialize model
		md=model;
		md=mesh(md,'DomainOutline.exp',9000);
		md=geography(md,'Iceshelves.exp','Islands.exp');
		md=parameterize(md,'Pig.par');
		md=extrude(md,4,2);
		md=setelementstype(md,'pattyn','all');
		if strcmpi(misfit,'absolute'),
			md.fit(:)=0;
		elseif strcmpi(misfit,'relative'),
			md.fit(:)=1;
		elseif strcmpi(misfit,'logarithmic'),
			md.fit(:)=2;
		end

		%launch control method
		[md packagefinal]=testsgetpackage(md,package);
		if ~testscheckmisfit(misfit), continue; end
			tic
		md=solve(md,'analysis_type','control','package',packagefinal);
		toc

		%compute fields to be checked
		fields=testsgetfields(md.type,'control');

		%load archive
		eval(['load Archive' package misfit ]);

		for k=1:length(fields),
			
			%Get field and tolerance
			field=fields{k};
			tolerance=testsgettolerance(md,package,misfit,field);

			%compare to archive
			eval(['Archive=Archive' package misfit '_field' num2str(k) ';']);
			eval(['error_diff=full(max(abs(Archive-md.results.' field '))/(max(abs(Archive))+eps));']);

			%disp test result
			if (error_diff>tolerance);
				disp(sprintf(['\n\nERROR   difference: %-7.2g > %7.2g test: %-25s   misfit: %-16s package: %-14s field: ' field  '.\n\n'],error_diff,tolerance,testname,misfit,package));
			else
				disp(sprintf(['\n\nSUCCESS difference: %-7.2g < %7.2g test: %-25s   misfit: %-16s package: %-14s field: ' field  '.\n\n'],error_diff,tolerance,testname,misfit,package));
			end

		end
	end
end
if nargout==1, 
	varargout{1}=md; 
end
